using Avalonia.Controls;
using Avalonia.Layout;
using Avalonia.Media;
using GLTFViewer.Engine.Ui;
using GLTFViewer.Engine.Ui.Controls;
using GLTFViewer.Rendering;

namespace GLTFViewer;

public partial class MainWindow : Window
{

    private Renderer _renderer;
    private SceneTree _sceneTree;
    
    private BaseSceneTreeControl _sceneTreeControl;
    private BaseOpenTKControl _openTkControl;
    private BasePropertyViewerControl _propertyViewerControl;
    
    public MainWindow()
    {
        InitializeComponent();
        
        _sceneTreeControl = SceneTreeControl;
        _openTkControl = OpenTkControl;
        _propertyViewerControl = PropertyViewerControl;
        
        _renderer = new Renderer();
        _sceneTree = new SceneTree(_sceneTreeControl.SceneTreeView);
        
        _sceneTreeControl.Init(_sceneTree);
        _propertyViewerControl.Init(_sceneTreeControl.SceneTreeView);
        
        _openTkControl.OnInit += _renderer.Initialize;
        _openTkControl.OnRender += _renderer.Render;
        _openTkControl.OnDeinit += _renderer.Deinitialize;
    }
}